﻿#include "I2Cdev.h"
#include "MPU6050_6Axis_MotionApps20.h"
#include <HMC5883L.h>
#include <Wire.h>
#include <Adafruit_GPS.h>

MPU6050 mpu;

#define LED_PIN 13
bool blinkState = false;
bool dmpReady = false;  // set true if DMP init was successful

uint8_t mpuIntStatus;   // holds actual interrupt status byte from MPU
uint8_t devStatus;      // return status after each device operation (0 = success, !0 = error)
uint16_t packetSize;    // expected DMP packet size (default is 42 bytes)
uint16_t fifoCount;     // count of all bytes currently in FIFO
uint8_t fifoBuffer[64]; // FIFO storage buffer

// orientation/motion vars
Quaternion q;           // [w, x, y, z]         quaternion container
VectorInt16 aa;         // [x, y, z]            accel sensor measurements
VectorInt16 aaReal;     // [x, y, z]            gravity-free accel sensor measurements
VectorInt16 aaWorld;    // [x, y, z]            world-frame accel sensor measurements
VectorFloat gravity;    // [x, y, z]            gravity vector

float euler[3];         // [psi, theta, phi]    Euler angle container
float ypr[3];           // [yaw, pitch, roll]   yaw/pitch/roll container and gravity vector

float Yaw;
float Pitch;
float Roll;
float Heading;

int Left = 0;
int Right = 0;
int Center = 0;

int FarLeft = 0;
int FarRight = 0;

int RearLeft = 0;
int RearRight = 0;
int Rear = 0;

float Latitude;
float Lat;

float Longitude;
float Lon;

float LatitudeDegrees;
float LongitudeDegrees;

float Speed;

int Disabled = 1;

volatile bool mpuInterrupt = false;     // indicates whether MPU interrupt pin has gone high

HardwareSerial mySerial = Serial2;

HMC5883L Compass;

Adafruit_GPS GPS(&mySerial);
boolean usingInterrupt = false;
void useInterrupt(boolean); // Func prototype keeps Arduino 0023 happy

uint32_t timer = millis();

String toWrite = "~000:000:000:000:000:000:000:000:0000:0000:0000:0000:0:000000:000000:000000:000000:0000:0000!";

void setup()
{
	useInterrupt(true); //GPS

	pinMode(28, INPUT); 	//sonar echo Center
	pinMode(26, OUTPUT);	//sonar trig

	pinMode(22, INPUT); 	//sonar echo brwn Right Front
	pinMode(24, OUTPUT);	//sonar trig red

	pinMode(37, INPUT); 	//sonar echo purple Left Front
	pinMode(35, OUTPUT);	//sonar trig blue

	pinMode(33, INPUT); 	//sonar echo gray Left
	pinMode(31, OUTPUT);	//sonar trig white

	pinMode(42, INPUT); 	//sonar echo Right
	pinMode(40, OUTPUT);	//sonar trig

	pinMode(38, INPUT); 	//sonar echo Right Rear
	pinMode(36, OUTPUT);	//sonar trig

	pinMode(51, INPUT); 	//sonar echo Left Rear
	pinMode(49, OUTPUT);	//sonar trig

	pinMode(45, INPUT); 	//sonar echo Rear
	pinMode(47, OUTPUT);	//sonar trig

	Wire.begin();
	TWBR = 24; // 400kHz I2C clock (200kHz if CPU is 8MHz)

	GPS.begin(9600);
	GPS.sendCommand(PMTK_SET_NMEA_OUTPUT_RMCONLY);//PMTK_SET_NMEA_OUTPUT_RMCONLY(PMTK_SET_NMEA_OUTPUT_RMCGGA
	GPS.sendCommand(PMTK_SET_NMEA_UPDATE_1HZ);

	Serial.begin(57600); //Debug
	Serial1.begin(57600); //PC data

	Compass.SetDeclination(8, 36, 'E');
	Compass.SetSamplingMode(COMPASS_SINGLE);
	Compass.SetScale(COMPASS_SCALE_130);
	Compass.SetOrientation(COMPASS_HORIZONTAL_X_NORTH);

	mpu.initialize();
	devStatus = mpu.dmpInitialize();

	// supply your own gyro offsets here, scaled for min sensitivity
	mpu.setXGyroOffset(220);
	mpu.setYGyroOffset(76);
	mpu.setZGyroOffset(-85);
	mpu.setZAccelOffset(1788); // 1688 factory default for my test chip
	mpu.setDMPEnabled(true);
	attachInterrupt(0, dmpDataReady, RISING);
	mpuIntStatus = mpu.getIntStatus();
	packetSize = mpu.dmpGetFIFOPacketSize();

	pinMode(LED_PIN, OUTPUT);
}

void loop()
{
	Center = Ping(26, 28, 74);
	Update();

	FarLeft = Ping(31, 33, 74);
	Update();

	Right = Ping(24, 22, 74);
	Update();

	Left = Ping(35, 37, 74);
	Update();

	RearLeft = Ping(49, 51, 74);
	Update();

	FarRight = Ping(40, 42, 74);
	Update();

	RearRight = Ping(36, 38, 74);
	Update();

	Rear = Ping(47, 45, 74);
	Update();

	// if a sentence is received, we can check the checksum, parse it...
	if (GPS.newNMEAreceived())
	{
		// a tricky thing here is if we print the NMEA sentence, or data
		// we end up not listening and catching other sentences! 
		// so be very wary if using OUTPUT_ALLDATA and trytng to print out data
		//Serial.println(GPS.lastNMEA());   // this also sets the newNMEAreceived() flag to false

		if (!GPS.parse(GPS.lastNMEA()))   // this also sets the newNMEAreceived() flag to false
			return;  // we can fail to parse a sentence in which case we should just wait for another
	}

	// if millis() or timer wraps around, we'll just reset it
	if (timer > millis())  timer = millis();

	if (millis() - timer > 2000)
	{
		timer = millis(); // reset the timer

		//Serial.print("\nTime: ");
		//Serial.print(GPS.hour, DEC); Serial.print(':');
		//Serial.print(GPS.minute, DEC); Serial.print(':');
		//Serial.print(GPS.seconds, DEC); Serial.print('.');
		//Serial.println(GPS.milliseconds);
		//Serial.print("Date: ");
		//Serial.print(GPS.day, DEC); Serial.print('/');
		//Serial.print(GPS.month, DEC); Serial.print("/20");
		//Serial.println(GPS.year, DEC);
		Serial.print("Fix: ");
		Serial.print((int)GPS.fix);
		Serial.println(" quality: ");
		Serial.println((int)GPS.fixquality);

		if (GPS.fix)
		{
			Latitude = GPS.latitude;
			Longitude = GPS.longitude;
			LatitudeDegrees = GPS.latitudeDegrees;
			LongitudeDegrees = GPS.longitudeDegrees;
			Speed = GPS.speed;
			Angle = GPS.angle;

			Serial.print("Location: ");
			Serial.print(Latitude, 4);
			Serial.print(GPS.lat);
			Serial.print(", ");
			Serial.print(Longitude, 4);
			Serial.println(GPS.lon);
			Serial.print("Location (in degrees, works with Google Maps): ");
			Serial.print(LatitudeDegrees, 4);
			Serial.print(", ");
			Serial.println(LongitudeDegrees, 4);

			Serial.print("Speed (knots): ");
			Serial.println(Speed);
			Serial.print("Angle: ");
			Serial.println(Angle);
			//Serial.print("Altitude: "); 
			//Serial.println(GPS.altitude);
			//Serial.print("Satellites: "); 
			//Serial.println((int)GPS.satellites);
		}
	}

	//blinkState = !blinkState;
	//digitalWrite(LED_PIN, blinkState);
}

// Interrupt is called once a millisecond, looks for any new GPS data, and stores it
SIGNAL(TIMER0_COMPA_vect)
{
	char c = GPS.read();
	// if you want to debug, this is a good time to do it!
#ifdef UDR0
	if (GPSECHO)
		if (c) UDR0 = c;
	// writing direct to UDR0 is much much faster than Serial.print 
	// but only one character can be written at a time. 
#endif
}

void useInterrupt(boolean v)
{
	if (v) {
		// Timer0 is already used for millis() - we'll just interrupt somewhere
		// in the middle and call the "Compare A" function above
		OCR0A = 0xAF;
		TIMSK0 |= _BV(OCIE0A);
		usingInterrupt = true;
	}
	else
	{
		// do not call the interrupt function COMPA anymore
		TIMSK0 &= ~_BV(OCIE0A);
		usingInterrupt = false;
	}
}

void UpdateYPRH()
{
	mpuInterrupt = false;

	mpuIntStatus = mpu.getIntStatus();

	fifoCount = mpu.getFIFOCount();

	while (fifoCount < packetSize) fifoCount = mpu.getFIFOCount();

	mpu.getFIFOBytes(fifoBuffer, packetSize);

	if ((mpuIntStatus & 0x10) || fifoCount == 1024)
	{
		mpu.resetFIFO();
	}
	else if (mpuIntStatus & 0x02)
	{
		mpu.dmpGetQuaternion(&q, fifoBuffer);
		mpu.dmpGetGravity(&gravity, &q);
		mpu.dmpGetYawPitchRoll(ypr, &q, &gravity);

		Yaw = ypr[0] * 180 / M_PI;
		Pitch = ypr[2] * 180 / M_PI;
		Roll = ypr[1] * 180 / M_PI;
	}

	Heading = Compass.GetHeadingDegrees();
}

void dmpDataReady()
{
	mpuInterrupt = true;
}

int Ping(int trigPin, int echoPin, int div)
{
	digitalWrite(trigPin, LOW);                           		// Set the trigger pin to low for 2uS
	delayMicroseconds(2);
	digitalWrite(trigPin, HIGH);                          		// Send a 10uS high to trigger ranging
	delayMicroseconds(10);
	digitalWrite(trigPin, LOW);                     		// Send pin low again
	return (pulseIn(echoPin, HIGH, 32000) / div / 2);
}

void Update()
{
	toWrite = "~";
	toWrite += String(Center);
	toWrite += ":" + String(Right);
	toWrite += ":" + String(Left);
	toWrite += ":" + String(FarRight);
	toWrite += ":" + String(FarLeft);
	toWrite += ":" + String(RearLeft);
	toWrite += ":" + String(RearRight)
	toWrite += ":" + String(Rear);
	toWrite += ":" + String(Yaw);
	toWrite += ":" + String(Pitch);
	toWrite += ":" + String(Roll);
	toWrite += ":" + String(Heading);
	toWrite += ":" + String(Disabled);
	toWrite += ":" + String(Latitude);
	toWrite += ":" + String(Longitude);
	toWrite += ":" + String(LatitudeDegrees);
	toWrite += ":" + String(LongitudeDegrees);
	toWrite += ":" + String(Speed);
	toWrite += ":" + String(Angle);
	toWrite += "!";

	Serial.println(toWrite); //Remove this when not debugging
	Serial1.println(toWrite);

	if (mpuInterrupt) UpdateYPRH();

	delay(40);
}